@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -17,7 +17,6 @@ from typing import Optional
 
 import click
 
-from spsdk import SPSDKError
 from spsdk.apps.blhost_helper import (
     PROPERTIES_OVERRIDE,
     OemGenMasterShareHelp,
@@ -26,31 +25,33 @@ from spsdk.apps.blhost_helper import (
     parse_property_tag,
     parse_trust_prov_key_type,
     parse_trust_prov_oem_key_type,
-    progress_bar,
 )
 from spsdk.apps.utils import spsdk_logger
 from spsdk.apps.utils.common_cli_options import (
     CommandsTreeGroup,
+    is_click_help,
     isp_interfaces,
     spsdk_apps_common_options,
+    spsdk_family_option,
 )
 from spsdk.apps.utils.utils import (
     INT,
     SPSDKAppError,
     catch_spsdk_error,
     format_raw_data,
-    get_interface,
-    get_key,
     parse_file_and_size,
     parse_hex_data,
+    progress_bar,
 )
-from spsdk.mboot import GenerateKeyBlobSelect, McuBoot, StatusCode, parse_property_value
+from spsdk.exceptions import SPSDKError
 from spsdk.mboot.error_codes import stringify_status_code
-from spsdk.utils.images import BinaryImage
+from spsdk.mboot.mcuboot import GenerateKeyBlobSelect, McuBoot, StatusCode, parse_property_value
+from spsdk.mboot.scanner import get_mboot_interface
+from spsdk.utils.misc import Endianness, load_hex_string
 
 
 @click.group(name="blhost", no_args_is_help=True, cls=CommandsTreeGroup)
-@isp_interfaces(uart=True, usb=True, sdio=True, lpcusbsio=True, buspal=True)
+@isp_interfaces(uart=True, usb=True, sdio=True, lpcusbsio=True, buspal=True, plugin=True)
 @spsdk_apps_common_options
 @click.option(
     "-s",
@@ -66,6 +67,7 @@ def main(
     sdio: str,
     buspal: str,
     lpcusbsio: str,
+    plugin: str,
     use_json: bool,
     log_level: int,
     timeout: int,
@@ -80,19 +82,23 @@ def main(
         args = sys.argv[1:]
         # running this via pytest changes the args to a single arg, in that case skip
         if len(args) > 1 and "get-property" in args:
-            tag_str = args[args.index("get-property") + 1]
-            if parse_property_tag(tag_str) == 0:
-                click.echo(ctx.command.commands["get-property"].help)  # type: ignore
-                ctx.exit(0)
+            try:
+                tag_str = args[args.index("get-property") + 1]
+                if parse_property_tag(tag_str) == 0:
+                    click.echo(ctx.command.commands["get-property"].help)  # type: ignore
+                    ctx.exit(0)
+            except IndexError:
+                click.echo("Invalid index for get-property", err=True)
+                ctx.exit(1)
 
     # if --help is provided anywhere on command line, skip interface lookup and display help message
-    if "--help" not in sys.argv[1:]:
+    if not is_click_help(ctx, sys.argv):
         ctx.obj = {
-            "interface": get_interface(
-                module="mboot",
+            "interface": get_mboot_interface(
                 port=port,
                 usb=usb,
                 sdio=sdio,
+                plugin=plugin,
                 timeout=timeout,
                 buspal=buspal,
                 lpcusbsio=lpcusbsio,
@@ -104,7 +110,7 @@ def main(
     return 0
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("command_file", type=click.Path(file_okay=True))
 @click.pass_context
 def batch(ctx: click.Context, command_file: str) -> None:
@@ -154,7 +160,7 @@ def call(ctx: click.Context, address: int, argument: int) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("memory_id", type=INT(), required=True)
 @click.argument("address", type=INT(), required=True)
 @click.pass_context
@@ -168,11 +174,11 @@ def configure_memory(ctx: click.Context, address: int, memory_id: int) -> None:
     ADDRESS     - starting address
     """
     with McuBoot(ctx.obj["interface"]) as mboot:
-        mboot.configure_memory(address, memory_id)  # type: ignore
+        mboot.configure_memory(address, memory_id)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("data", type=INT(base=16), required=True)
 @click.argument(
@@ -185,8 +191,8 @@ def configure_memory(ctx: click.Context, address: int, memory_id: int) -> None:
     "-v / ",
     "--verify/--no-verify",
     is_flag=True,
-    default=True,
-    help="Verify write operation (verify by default)",
+    default=False,
+    help="Verify write operation (don't verify by default)",
 )
 @click.pass_context
 def efuse_program_once(
@@ -207,7 +213,7 @@ def efuse_program_once(
         display_output([response], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.pass_context
 def efuse_read_once(ctx: click.Context, address: int) -> None:
@@ -226,7 +232,7 @@ def efuse_read_once(ctx: click.Context, address: int) -> None:
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("argument", type=INT(), required=True)
 @click.argument("stackpointer", type=INT(), required=True)
@@ -250,7 +256,7 @@ def execute(ctx: click.Context, address: int, argument: int, stackpointer: int)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("byte_count", type=INT(), required=True)
 @click.argument("memory_id", type=INT(), required=False, default="0")
@@ -297,7 +303,7 @@ def flash_erase_all_unsecure(ctx: click.Context) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("image_file_path", metavar="FILE", type=str, required=True)
 @click.argument("erase", type=str, required=False, default="none")
 @click.argument("memory_id", type=INT(), required=False, default="0")
@@ -310,6 +316,8 @@ def flash_image(ctx: click.Context, image_file_path: str, erase: str, memory_id:
     ERASE      - string 'erase' determines if flash is erased before writing
     MEMORY_ID  - id of memory to erase (default: 0)
     """
+    from spsdk.utils.images import BinaryImage
+
     ALIGNMENT = 1024
 
     if not os.path.isfile(image_file_path):
@@ -349,7 +357,7 @@ def flash_image(ctx: click.Context, image_file_path: str, erase: str, memory_id:
             display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("index", type=INT(), required=True)
 @click.argument("byte_count", type=click.Choice(["4", "8"]), required=True)
 @click.argument("data", type=INT(base=16), required=True)
@@ -372,16 +380,16 @@ def flash_program_once(
     DATA        - 4 or 8-byte-hex according to <byte_count>
     ENDIANNESS   - output sequence is specified by LSB (Default) or MSB
     """
-    byte_order = "big" if endianness == "MSB" else "little"
-    input_data = data.to_bytes(int(byte_count), byteorder=byte_order)  # type: ignore[arg-type]
+    byte_order = Endianness.BIG if endianness == "MSB" else Endianness.LITTLE
+    input_data = data.to_bytes(int(byte_count), byteorder=byte_order.value)
     with McuBoot(ctx.obj["interface"]) as mboot:
         mboot.flash_program_once(index=index, data=input_data)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("index", type=INT(), required=True)
-@click.argument("byte_count", type=click.Choice(["4", "8"]), required=True, default=4)
+@click.argument("byte_count", type=click.Choice(["4", "8"]), required=True, default="4")
 @click.pass_context
 def flash_read_once(ctx: click.Context, index: int, byte_count: str) -> None:
     """Returns the contents of a specific program once field.
@@ -393,14 +401,18 @@ def flash_read_once(ctx: click.Context, index: int, byte_count: str) -> None:
     with McuBoot(ctx.obj["interface"]) as mboot:
         response = mboot.flash_read_once(index=index, count=int(byte_count))
         display_output(
-            None if response is None else [len(response), int.from_bytes(response, "little")],
+            (
+                None
+                if response is None
+                else [len(response), int.from_bytes(response, Endianness.LITTLE.value)]
+            ),
             mboot.status_code,
             ctx.obj["use_json"],
             ctx.obj["silent"],
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("key", type=str, required=True)
 @click.pass_context
 def flash_security_disable(ctx: click.Context, key: str) -> None:
@@ -420,7 +432,7 @@ def flash_security_disable(ctx: click.Context, key: str) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("length", type=INT(), required=True)
 @click.argument("option", type=click.Choice(["0", "1"]), required=True)
@@ -464,7 +476,7 @@ def flash_read_resource(
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("byte_count", type=INT(), required=True)
 @click.argument("pattern", type=INT(), required=True)
@@ -493,7 +505,7 @@ def fill_memory(
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("data_source", metavar="FILE[,BYTE_COUNT] | {{HEX-DATA}}", type=str, required=True)
 @click.argument("memory_id", type=INT(), required=False, default="0")
@@ -526,7 +538,7 @@ def fuse_program(ctx: click.Context, address: int, data_source: str, memory_id:
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("byte_count", type=INT(), required=True)
 @click.argument("out_file", metavar="FILE", type=click.File("wb"), required=False)
@@ -588,7 +600,7 @@ def list_memory(ctx: click.Context) -> None:
             print(f"{(mem.name)}:\n  {mem}")
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("boot_file", metavar="FILE", type=click.File("rb"))
 @click.pass_context
 def load_image(ctx: click.Context, boot_file: click.File) -> None:
@@ -609,15 +621,10 @@ def load_image(ctx: click.Context, boot_file: click.File) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("property_tag", type=str, required=True)
 @click.argument("index", type=INT(), default="0")
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(list(PROPERTIES_OVERRIDE.keys())),
-    help="Provide family. So far supported 'kw45xx' and 'k32w1xx'",
-)
+@spsdk_family_option(families=list(PROPERTIES_OVERRIDE.keys()), required=False)
 @click.pass_context
 def get_property(
     ctx: click.Context,
@@ -676,11 +683,11 @@ def get_property(
     \b
     Note: Not all the properties are available for all devices.
     """
-    property_tag_int = parse_property_tag(property_tag, family)
+    property_tag_enum = parse_property_tag(property_tag, family)
     with McuBoot(ctx.obj["interface"]) as mboot:
-        response = mboot.get_property(property_tag_int, index=index)  # type: ignore
+        response = mboot.get_property(property_tag_enum, index=index)
         property_text = (
-            str(parse_property_value(property_tag_int, response, None, family))
+            str(parse_property_value(property_tag_enum.tag, response, None, family))
             if response
             else None
         )
@@ -689,15 +696,10 @@ def get_property(
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("property_tag", type=str, required=True)
 @click.argument("value", type=INT(), required=True)
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(list(PROPERTIES_OVERRIDE.keys())),
-    help="Provide family. So far supported 'kw45xx' and 'k32w1xx'",
-)
+@spsdk_family_option(families=list(PROPERTIES_OVERRIDE.keys()), required=False)
 @click.pass_context
 def set_property(
     ctx: click.Context,
@@ -730,11 +732,11 @@ def set_property(
     """
     property_tag_int = parse_property_tag(property_tag, family)
     with McuBoot(ctx.obj["interface"]) as mboot:
-        mboot.set_property(prop_tag=property_tag_int, value=value)  # type: ignore
+        mboot.set_property(prop_tag=property_tag_int, value=value)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("byte_count", type=INT(), required=True)
 @click.argument("out_file", metavar="FILE", type=click.File("wb"), required=False)
@@ -792,7 +794,7 @@ def read_memory(
     )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("sb_file", metavar="FILE", type=click.File("rb"), required=True)
 @click.option(
     "-c",
@@ -826,7 +828,7 @@ def receive_sb_file(ctx: click.Context, sb_file: click.File, check_errors: bool)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.pass_context
 def reliable_update(ctx: click.Context, address: int) -> None:
@@ -852,7 +854,7 @@ def reset(ctx: click.Context) -> None:
     display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("address", type=INT(), required=True)
 @click.argument("data_source", metavar="FILE[,BYTE_COUNT] | {{HEX-DATA}}", type=str, required=True)
 @click.argument("memory_id", type=INT(), required=False, default="0")
@@ -891,7 +893,7 @@ def write_memory(ctx: click.Context, address: int, data_source: str, memory_id:
         )
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("dek_file", type=click.File("rb"), required=True)
 @click.argument("blob_file", type=click.File("wb"), required=True)
 @click.argument(
@@ -920,7 +922,9 @@ def generate_key_blob(
     """
     with McuBoot(ctx.obj["interface"]) as mboot:
         data = dek_file.read()  # type: ignore
-        key_sel_int = int(key_sel) if key_sel.isnumeric() else GenerateKeyBlobSelect.get(key_sel)
+        key_sel_int = (
+            int(key_sel) if key_sel.isnumeric() else GenerateKeyBlobSelect.get_tag(key_sel)
+        )
         assert isinstance(key_sel_int, int)
         write_response = mboot.generate_key_blob(data, key_sel=key_sel_int)
         if write_response:
@@ -939,7 +943,7 @@ def key_provisioning(ctx: click.Context) -> None:  # pylint: disable=unused-argu
     """Group of sub-commands related to key provisioning."""
 
 
-@key_provisioning.command()
+@key_provisioning.command(no_args_is_help=False)
 @click.pass_context
 def enroll(ctx: click.Context) -> None:
     """Enrolls key provisioning feature. No argument for this operation."""
@@ -948,7 +952,7 @@ def enroll(ctx: click.Context) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("file", metavar="FILE", type=click.File("rb"), required=True)
 @click.pass_context
 def program_aeskey(ctx: click.Context, file: click.File) -> None:
@@ -963,7 +967,7 @@ def program_aeskey(ctx: click.Context, file: click.File) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@key_provisioning.command(name="set_user_key")
+@key_provisioning.command(name="set_user_key", no_args_is_help=True)
 @click.option(
     "-s",
     "--key-size",
@@ -1018,14 +1022,14 @@ def set_user_key(ctx: click.Context, key_type: str, file_and_size: str, key_size
     if not key_size:
         key_size = (size if size > 0 else os.path.getsize(file_path)) * 8
 
-    key_data = get_key(file_path, expected_size=key_size // 8)
+    key_data = load_hex_string(file_path, expected_size=key_size // 8)
 
     with McuBoot(ctx.obj["interface"]) as mboot:
-        mboot.kp_set_user_key(key_type=key_type_int, key_data=key_data)  # type: ignore
+        mboot.kp_set_user_key(key_type=key_type_int, key_data=key_data)
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@key_provisioning.command(name="set_key")
+@key_provisioning.command(name="set_key", no_args_is_help=True)
 @click.argument("key_type", metavar="TYPE", type=str, required=True)
 @click.argument("key_size", metavar="SIZE", type=INT(), required=True)
 @click.pass_context
@@ -1085,7 +1089,7 @@ def read_key_nonvolatile(ctx: click.Context, memory_id: int) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@key_provisioning.command(name="write_key_store")
+@key_provisioning.command(name="write_key_store", no_args_is_help=True)
 @click.argument("file_and_size", metavar="FILE[,SIZE]", type=str, required=True)
 @click.pass_context
 def write_key_store(ctx: click.Context, file_and_size: str) -> None:
@@ -1110,7 +1114,7 @@ def write_key_store(ctx: click.Context, file_and_size: str) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@key_provisioning.command(name="read_key_store")
+@key_provisioning.command(name="read_key_store", no_args_is_help=True)
 @click.argument("key_store_file", metavar="FILE", type=click.File("wb"), required=True)
 @click.pass_context
 def read_key_store(ctx: click.Context, key_store_file: click.File) -> None:
@@ -1139,7 +1143,7 @@ def trust_provisioning(ctx: click.Context) -> None:  # pylint: disable=unused-ar
     """Group of sub-commands related to trust provisioning."""
 
 
-@trust_provisioning.command(name="hsm_store_key")
+@trust_provisioning.command(name="hsm_store_key", no_args_is_help=True)
 @click.argument("key_type", metavar="KEY_TYPE", type=str, required=True)
 @click.argument("key_property", metavar="KEY_PROPERTY", type=INT(), required=True)
 @click.argument("key_input_addr", metavar="KEY_INPUT_ADDR", type=INT(), required=True)
@@ -1195,7 +1199,7 @@ def hsm_store_key(
         )
 
 
-@trust_provisioning.command(name="hsm_gen_key")
+@trust_provisioning.command(name="hsm_gen_key", no_args_is_help=True)
 @click.argument("key_type", metavar="KEY_TYPE", type=str, required=True)
 @click.argument("reserved", metavar="RESERVED", type=INT(), required=True)
 @click.argument("key_blob_output_addr", metavar="KEY_BLOB_OUTPUT_ADDR", type=INT(), required=True)
@@ -1255,7 +1259,7 @@ def hsm_gen_key(
         )
 
 
-@trust_provisioning.command(name="hsm_enc_blk")
+@trust_provisioning.command(name="hsm_enc_blk", no_args_is_help=True)
 @click.argument(
     "mfg_cust_mk_sk_0_blob_input_addr",
     metavar="MFG_CUST_MK_SK_0_BLOB_INPUT_ADDR",
@@ -1315,7 +1319,7 @@ def hsm_enc_blk(
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@trust_provisioning.command(name="hsm_enc_sign")
+@trust_provisioning.command(name="hsm_enc_sign", no_args_is_help=True)
 @click.argument("key_blob_input_addr", metavar="KEY_BLOB_INPUT_ADDR", type=INT(), required=True)
 @click.argument("key_blob_input_size", metavar="KEY_BLOB_INPUT_SIZE", type=INT(), required=True)
 @click.argument("block_data_input_addr", metavar="BLOCK_DATA_INPUT_ADDR", type=INT(), required=True)
@@ -1371,7 +1375,11 @@ def hsm_enc_sign(
         )
 
 
-@trust_provisioning.command(name="oem_gen_master_share", cls=OemGenMasterShareHelp)
+@trust_provisioning.command(
+    name="oem_gen_master_share",
+    cls=OemGenMasterShareHelp,
+    no_args_is_help=True,
+)
 @click.argument("oem_share_input_addr", type=INT(), required=True)
 @click.argument("oem_share_input_size", type=INT(), required=True)
 @click.argument("oem_enc_share_output_addr", type=INT(), required=True)
@@ -1437,7 +1445,9 @@ def oem_gen_master_share(
         )
 
 
-@trust_provisioning.command(name="oem_set_master_share", cls=OemSetMasterShareHelp)
+@trust_provisioning.command(
+    name="oem_set_master_share", cls=OemSetMasterShareHelp, no_args_is_help=True
+)
 @click.argument("oem_share_input_addr", type=INT(), required=True)
 @click.argument("oem_share_input_size", type=INT(), required=True)
 @click.argument("oem_enc_master_share_input_addr", type=INT(), required=True)
@@ -1461,7 +1471,7 @@ def oem_set_master_share(
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@trust_provisioning.command(name="oem_get_cust_cert_dice_puk")
+@trust_provisioning.command(name="oem_get_cust_cert_dice_puk", no_args_is_help=True)
 @click.argument("oem_rkth_input_addr", type=INT(), required=True)
 @click.argument("oem_rkth_input_size", type=INT(), required=True)
 @click.argument("oem_cust_cert_dice_puk_output_addr", type=INT(), required=True)
@@ -1505,7 +1515,223 @@ def oem_get_cust_cert_dice_puk(
     )
 
 
-@main.command()
+@trust_provisioning.command(name="wpc_get_id", no_args_is_help=True)
+@click.argument("wpc_id_blob_addr", type=INT(), required=True)
+@click.argument("wpc_id_blob_size", type=INT(), required=True)
+@click.pass_context
+def wpc_get_id(
+    ctx: click.Context,
+    wpc_id_blob_addr: int,
+    wpc_id_blob_size: int,
+) -> None:
+    """Command used for harvesting device ID blob.
+
+    \b
+    WPC_ID_BLOB_ADDR - Buffer address
+    WPC_ID_BLOB_SIZE - Buffer size
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.wpc_get_id(
+            wpc_id_blob_addr,
+            wpc_id_blob_size,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="nxp_get_id", no_args_is_help=True)
+@click.argument("id_blob_addr", type=INT(), required=True)
+@click.argument("id_blob_size", type=INT(), required=True)
+@click.pass_context
+def nxp_get_id(
+    ctx: click.Context,
+    id_blob_addr: int,
+    id_blob_size: int,
+) -> None:
+    """Command used for harvesting device ID blob during wafer test as part of RTS flow.
+
+    This command is allowed only when LC_STATE < 0x3.
+
+    \b
+    ID_BLOB_ADDR            - address of ID blob defined by Round-trip trust provisioning specification.
+    ID_BLOB_SIZE            - length of buffer in bytes
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.nxp_get_id(
+            id_blob_addr,
+            id_blob_size,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="wpc_insert_cert", no_args_is_help=True)
+@click.argument("wpc_cert_addr", type=INT(), required=True)
+@click.argument("wpc_cert_len", type=INT(), required=True)
+@click.argument("ec_id_offset", type=INT(), required=True)
+@click.argument("wpc_puk_offset", type=INT(), required=True)
+@click.pass_context
+def wpc_insert_cert(
+    ctx: click.Context,
+    wpc_cert_addr: int,
+    wpc_cert_len: int,
+    ec_id_offset: int,
+    wpc_puk_offset: int,
+) -> None:
+    """Command used for certificate validation before it is written into flash.
+
+    This command does following things:
+        Extracts ECID and WPC PUK from certificate
+        Validates ECID and WPC PUK. If both are OK it returns success. Otherwise returns fail
+
+    \b
+    WPC_CERT_ADDR   - address of inserted certificate
+    WPC_CERT_LEN    - length in bytes of inserted certificate
+    EC_ID_OFFSET    - offset to 72-bit ECID
+    WPC_PUK_OFFSET  - WPC PUK offset from beginning of inserted certificate
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.wpc_insert_cert(
+            wpc_cert_addr,
+            wpc_cert_len,
+            ec_id_offset,
+            wpc_puk_offset,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="wpc_sign_csr", no_args_is_help=True)
+@click.argument("csr_tbs_addr", type=INT(), required=True)
+@click.argument("csr_tbs_len", type=INT(), required=True)
+@click.argument("signature_addr", type=INT(), required=True)
+@click.argument("signature_len", type=INT(), required=True)
+@click.pass_context
+def wpc_sign_csr(
+    ctx: click.Context,
+    csr_tbs_addr: int,
+    csr_tbs_len: int,
+    signature_addr: int,
+    signature_len: int,
+) -> None:
+    """Command used for signing CSR data (TBS portion).
+
+    \b
+    CSR_TBS_ADDR   - address of CSR-TBS data
+    CSR_TBS_LEN    - length in bytes of CSR-TBS data
+    SIGNATURE_ADDR - address where to store signature
+    SIGNATURE_LEN  - expected length of signature
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.wpc_sign_csr(
+            csr_tbs_addr,
+            csr_tbs_len,
+            signature_addr,
+            signature_len,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="dsc_hsm_create_session", no_args_is_help=True)
+@click.argument("oem_seed_input_addr", type=INT(), required=True)
+@click.argument("oem_seed_input_size", type=INT(), required=True)
+@click.argument("oem_share_output_addr", type=INT(), required=True)
+@click.argument("oem_share_output_size", type=INT(), required=True)
+@click.pass_context
+def dsc_hsm_create_session(
+    ctx: click.Context,
+    oem_seed_input_addr: int,
+    oem_seed_input_size: int,
+    oem_share_output_addr: int,
+    oem_share_output_size: int,
+) -> None:
+    """Command used by OEM to provide it share to create the initial trust provisioning keys.
+
+    \b
+    OEM_SEED_INPUT_ADDR     - address of 128-bit entropy seed value provided by the OEM.
+    OEM_SEED_INPUT_SIZE     - OEM seed size in bytes
+    OEM_SHARE_OUTPUT_ADDR   - A 128-bit encrypted token.
+    OEM_SHARE_OUTPUT_SIZE   - size in bytes
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.dsc_hsm_create_session(
+            oem_seed_input_addr,
+            oem_seed_input_size,
+            oem_share_output_addr,
+            oem_share_output_size,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="dsc_hsm_enc_blk", no_args_is_help=True)
+@click.argument("sbx_header_input_addr", type=INT(), required=True)
+@click.argument("sbx_header_input_size", type=INT(), required=True)
+@click.argument("block_num", type=INT(), required=True)
+@click.argument("block_data_addr", type=INT(), required=True)
+@click.argument("block_data_size", type=INT(), required=True)
+@click.pass_context
+def dsc_hsm_enc_blk(
+    ctx: click.Context,
+    sbx_header_input_addr: int,
+    sbx_header_input_size: int,
+    block_num: int,
+    block_data_addr: int,
+    block_data_size: int,
+) -> None:
+    """Command used to encrypt the given block sliced by the nxpimage.
+
+    This command is only supported after issuance of dsc_hsm_create_session.
+
+    \b
+    SBX_HEADER_INPUT_ADDR - SBx header containing file size, Firmware version and Timestamp data.
+         Except for hash digest of block 0, all other fields should be valid.
+    SBX_HEADER_INPUT_SIZE - size of the header in bytes
+    BLOCK_NUM             - Number of block
+    BLOCK_DATA_ADDR       - Address of data block
+    BLOCK_DATA_SIZE       - Size of data block
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.dsc_hsm_enc_blk(
+            sbx_header_input_addr,
+            sbx_header_input_size,
+            block_num,
+            block_data_addr,
+            block_data_size,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@trust_provisioning.command(name="dsc_hsm_enc_sign", no_args_is_help=True)
+@click.argument("block_data_input_addr", type=INT(), required=True)
+@click.argument("block_data_input_size", type=INT(), required=True)
+@click.argument("signature_output_addr", type=INT(), required=True)
+@click.argument("block_data_addr", type=INT(), required=True)
+@click.pass_context
+def dsc_hsm_enc_sign(
+    ctx: click.Context,
+    block_data_input_addr: int,
+    block_data_input_size: int,
+    signature_output_addr: int,
+    signature_output_size: int,
+) -> None:
+    """Command used for signing the data buffer provided.
+
+    This command is only supported after issuance of dsc_hsm_create_session.
+
+    \b
+    BLOCK_DATA_INPUT_ADDR - Address of data buffer to be signed
+    BLOCK_DATA_INPUT_SIZE - Size of data buffer in bytes
+    SIGNATURE_OUTPUT_ADDR - Addres to output signature data
+    SIGNATURE_OUTPUT_SIZE - Size of the output signature data in bytes
+    """
+    with McuBoot(ctx.obj["interface"]) as mboot:
+        mboot.dsc_hsm_enc_sign(
+            block_data_input_addr,
+            block_data_input_size,
+            signature_output_addr,
+            signature_output_size,
+        )
+        display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
+
+
+@main.command(no_args_is_help=True)
 @click.argument("life-cycle", metavar="LIFE CYCLE", type=INT(), required=True)
 @click.pass_context
 def update_life_cycle(ctx: click.Context, life_cycle: int) -> None:
@@ -1519,7 +1745,7 @@ def update_life_cycle(ctx: click.Context, life_cycle: int) -> None:
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@main.command()
+@main.command(no_args_is_help=True)
 @click.argument("cmd-msg-addr", metavar="COMMAND MESSAGE ADDRESS", type=INT(), required=True)
 @click.argument("cmd-msg-cnt", metavar="COMMAND MESSAGE COUNT", type=INT(), required=True)
 @click.argument("resp-msg-addr", metavar="RESPONSE MESSAGE ADDRESS", type=INT(), required=True)
@@ -1554,7 +1780,7 @@ def ele_message(
         display_output([], mboot.status_code, ctx.obj["use_json"], ctx.obj["silent"])
 
 
-@trust_provisioning.command(name="prove_genuinity")
+@trust_provisioning.command(name="prove_genuinity", no_args_is_help=True)
 @click.argument("address", type=INT())
 @click.argument("buffer_size", type=INT())
 @click.pass_context
@@ -1571,13 +1797,15 @@ def prove_genuinity(ctx: click.Context, address: int, buffer_size: int) -> None:
             [tp_response_length],
             mboot.status_code,
             use_json=ctx.obj["use_json"],
-            extra_output=f"TP response will be {tp_response_length} bytes long."
-            if tp_response_length
-            else None,
+            extra_output=(
+                f"TP response will be {tp_response_length} bytes long."
+                if tp_response_length
+                else None
+            ),
         )
 
 
-@trust_provisioning.command(name="isp_set_wrap_data")
+@trust_provisioning.command(name="isp_set_wrap_data", no_args_is_help=True)
 @click.argument("address", type=INT())
 @click.argument("control", type=INT(), required=False, default=0x1)
 @click.argument("stage", type=INT(), required=False, default=0x4B)
