@@ -1,20 +1,23 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for DebugMailbox Debug probes support."""
 
 import functools
 import logging
+from abc import ABC, abstractmethod
 from time import sleep
-from typing import Any, Dict, List, Optional, no_type_check
+from typing import Any, Dict, List, Optional, Type, no_type_check
 
-from spsdk import SPSDKError
-from spsdk.exceptions import SPSDKValueError
+import colorama
+import prettytable
+
+from spsdk.exceptions import SPSDKError, SPSDKValueError
 from spsdk.utils.exceptions import SPSDKTimeoutError
-from spsdk.utils.misc import Timeout
+from spsdk.utils.misc import Timeout, value_to_int
 
 logger = logging.getLogger(__name__)
 
@@ -38,9 +41,11 @@ class SPSDKDebugProbeNotOpenError(SPSDKDebugProbeError):
     """The debug probe is not opened exception for use with SPSDK."""
 
 
-class DebugProbe:
+class DebugProbe(ABC):
     """Abstraction class to define SPSDK debug probes interface."""
 
+    NAME = "Abstract"
+
     # Constants to detect the debug mailbox access port
     APBANKSEL = 0x000000F0
     APBANK_SHIFT = 4
@@ -59,9 +64,14 @@ class DebugProbe:
     CDBGPWRUPREQ = 0x10 << 24
     MASKLANE = 0x0F << 8
 
+    # Constants for DHCSR, Debug Halting Control and Status Register
+    DHCSR_REG = 0xE000EDF0
+    DHCSR_DEBUGKEY = 0xA05F0000
+    DHCSR_C_DEBUGEN = 0x1
+    DHCSR_C_HALT = 0x2
+
     RESET_TIME = 0.1
     AFTER_RESET_TIME = 0.05
-    TEST_MEM_AP_ADDRESS = 0x2000_8000
 
     def __init__(self, hardware_id: str, options: Optional[Dict] = None) -> None:
         """This is general initialization function for SPSDK library to support various DEBUG PROBES.
@@ -71,14 +81,13 @@ class DebugProbe:
         """
         self.hardware_id = hardware_id
         self.options = options or {}
-        self.disable_reinit = False
-        self.last_accessed_ap = -1
         self.mem_ap_ix = -1
 
     @classmethod
+    @abstractmethod
     def get_connected_probes(
         cls, hardware_id: Optional[str] = None, options: Optional[Dict] = None
-    ) -> list:
+    ) -> "DebugProbes":
         """Functions returns the list of all connected probes in system.
 
         There is option to look for just for one debug probe defined by its hardware ID.
@@ -86,10 +95,8 @@ class DebugProbe:
         :param hardware_id: None to list all probes, otherwise the the only probe with
             matching hardware id is listed.
         :param options: The options dictionary
-        :return: ProbeDescription
-        :raises NotImplementedError: Derived class has to implement this method
+        :return: List of ProbeDescription
         """
-        raise NotImplementedError("Derived class has to implement this method.")
 
     @staticmethod
     def get_options_help() -> Dict[str, str]:
@@ -97,27 +104,117 @@ class DebugProbe:
 
         :return: Dictionary with individual options. Key is parameter name and value the help text.
         """
-        return {}
+        return {
+            "test_address": "Address for testing memory AP, default is 0x2000_0000",
+        }
 
+    @staticmethod
+    def get_coresight_ap_address(access_port: int, address: int) -> int:
+        """Return computed address of coresight access port register.
+
+        :param access_port: Index of access port 0-255.
+        :param address: Register address.
+        :return: Coresight address.
+        :raises SPSDKError: In case of invalid value.
+        """
+        if access_port > 255:
+            raise SPSDKValueError("Invalid value of access port")
+
+        return access_port << DebugProbe.APSEL_SHIFT | address
+
+    @abstractmethod
     def open(self) -> None:
         """Debug probe open.
 
         General opening function for SPSDK library to support various DEBUG PROBES.
         The function is used to initialize the connection to target and enable using debug probe
         for DAT purposes.
-
-        :raises NotImplementedError: Derived class has to implement this method
         """
-        raise NotImplementedError("Derived class has to implement this method.")
 
+    @abstractmethod
     def close(self) -> None:
         """Debug probe close.
 
         This is general closing function for SPSDK library to support various DEBUG PROBES.
-
-        :raises NotImplementedError: Derived class has to implement this method
         """
 
+    @abstractmethod
+    def mem_reg_read(self, addr: int = 0) -> int:
+        """Read 32-bit register in memory space of MCU.
+
+        This is read 32-bit register in memory space of MCU function for SPSDK library
+        to support various DEBUG PROBES.
+
+        :param addr: The register address
+        :return: The read value of addressed register (4 bytes)
+        """
+
+    @abstractmethod
+    def mem_reg_write(self, addr: int = 0, data: int = 0) -> None:
+        """Write 32-bit register in memory space of MCU.
+
+        This is write 32-bit register in memory space of MCU function for SPSDK library
+        to support various DEBUG PROBES.
+
+        :param addr: the register address
+        :param data: the data to be written into register
+        """
+
+    @abstractmethod
+    def coresight_reg_read(self, access_port: bool = True, addr: int = 0) -> int:
+        """Read coresight register.
+
+        It reads coresight register function for SPSDK library to support various DEBUG PROBES.
+
+        :param access_port: if True, the Access Port (AP) register will be read(default), otherwise the Debug Port
+        :param addr: the register address
+        :return: The read value of addressed register (4 bytes)
+        """
+
+    @abstractmethod
+    def coresight_reg_write(self, access_port: bool = True, addr: int = 0, data: int = 0) -> None:
+        """Write coresight register.
+
+        It writes coresight register function for SPSDK library to support various DEBUG PROBES.
+
+        :param access_port: if True, the Access Port (AP) register will be write(default), otherwise the Debug Port
+        :param addr: the register address
+        :param data: the data to be written into register
+        """
+
+    @abstractmethod
+    def assert_reset_line(self, assert_reset: bool = False) -> None:
+        """Control reset line at a target.
+
+        :param assert_reset: If True, the reset line is asserted(pulled down), if False the reset line is not affected.
+        """
+
+    def reset(self) -> None:
+        """Reset a target.
+
+        It resets a target.
+        """
+        self.assert_reset_line(True)
+        sleep(self.RESET_TIME)
+        self.assert_reset_line(False)
+        sleep(self.AFTER_RESET_TIME)
+
+
+class DebugProbeLocal(DebugProbe):
+    """Abstraction class to define SPSDK debug probes interface."""
+
+    NAME = "local_help"
+
+    def __init__(self, hardware_id: str, options: Optional[Dict[str, str]] = None) -> None:
+        """This is general initialization function for SPSDK library to support various DEBUG PROBES.
+
+        :param hardware_id: Open probe with selected hardware ID
+        :param options: The options dictionary
+        """
+        super().__init__(hardware_id, options)
+        self.disable_reinit = False
+        self.last_accessed_ap = -1
+
     @no_type_check
     # pylint: disable=no-self-argument
     def get_mem_ap(func: Any) -> Any:
@@ -126,9 +223,15 @@ class DebugProbe:
         :param func: Decorated function.
         """
         POSSIBLE_MEM_AP_IX = [0, 1, 3]
+        DEFAULT_TEST_MEM_AP_ADDRESS = 0x2000_0000
 
         @functools.wraps(func)
-        def wrapper(self: "DebugProbe", *args, **kwargs) -> Any:
+        def wrapper(self: "DebugProbeLocal", *args, **kwargs) -> Any:
+            status = False
+            test_address = value_to_int(
+                self.options.get("test_address", DEFAULT_TEST_MEM_AP_ADDRESS)
+            )
+
             if self.mem_ap_ix < 0:
                 # Try to find MEM AP
                 for i in POSSIBLE_MEM_AP_IX:
@@ -141,7 +244,37 @@ class DebugProbe:
                         ap_class = (idr & 0x1E000) >> 13
                         if ap_class == 8:
                             try:
-                                self._mem_reg_read(mem_ap_ix=i, addr=self.TEST_MEM_AP_ADDRESS)
+                                # Enter debug state and halt
+                                dhcsr_reg = self._mem_reg_read(mem_ap_ix=i, addr=self.DHCSR_REG)
+                                logger.debug(f"Value of DHCSR register = {hex(dhcsr_reg)}")
+                                self._mem_reg_write(
+                                    mem_ap_ix=i,
+                                    addr=self.DHCSR_REG,
+                                    data=(
+                                        self.DHCSR_DEBUGKEY
+                                        | self.DHCSR_C_HALT
+                                        | self.DHCSR_C_DEBUGEN
+                                    ),
+                                )
+                                try:
+                                    self._mem_reg_read(mem_ap_ix=i, addr=test_address)
+                                    status = True
+                                except SPSDKError:
+                                    logger.debug(
+                                        f"Read operation on AP{i} fails at {hex(test_address)} address"
+                                    )
+                                finally:
+                                    # Exit debug state
+                                    self._mem_reg_write(
+                                        mem_ap_ix=i,
+                                        addr=self.DHCSR_REG,
+                                        data=(self.DHCSR_DEBUGKEY | self.DHCSR_C_DEBUGEN),
+                                    )
+                                    self._mem_reg_write(
+                                        mem_ap_ix=i, addr=self.DHCSR_REG, data=self.DHCSR_DEBUGKEY
+                                    )
+                                if not status:
+                                    continue
                             except SPSDKError:
                                 continue
 
@@ -200,6 +333,38 @@ class DebugProbe:
         """
         return self._mem_reg_read(mem_ap_ix=self.mem_ap_ix, addr=addr)
 
+    def _mem_reg_write(self, mem_ap_ix: int, addr: int = 0, data: int = 0) -> None:
+        """Write 32-bit register in memory space of MCU.
+
+        This is write 32-bit register in memory space of MCU function for SPSDK library
+        to support various DEBUG PROBES.
+
+        :param mem_ap_ix: The index of memory access port
+        :param addr: the register address
+        :param data: the data to be written into register
+        :raises SPSDKDebugProbeTransferError: Error occur during memory transfer.
+        """
+        try:
+            self.coresight_reg_write(
+                access_port=True,
+                addr=self.get_coresight_ap_address(mem_ap_ix, 0 * 4),
+                data=0x22000012,
+            )
+            self.coresight_reg_write(
+                access_port=True,
+                addr=self.get_coresight_ap_address(mem_ap_ix, 1 * 4),
+                data=addr,
+            )
+            self.coresight_reg_write(
+                access_port=True,
+                addr=self.get_coresight_ap_address(mem_ap_ix, 3 * 4),
+                data=data,
+            )
+            self.coresight_reg_read(access_port=False, addr=self.DP_CTRL_STAT_REG)
+        except SPSDKError as exc:
+            self.clear_sticky_errors()
+            raise SPSDKDebugProbeTransferError(f"Failed write memory({str(exc)})") from exc
+
     @get_mem_ap
     def mem_reg_write(self, addr: int = 0, data: int = 0) -> None:
         """Write 32-bit register in memory space of MCU.
@@ -209,68 +374,14 @@ class DebugProbe:
 
         :param addr: the register address
         :param data: the data to be written into register
-        :raises SPSDKDebugProbeTransferError: Error occur during memory transfer.
         """
-        try:
-            self.coresight_reg_write(
-                access_port=True,
-                addr=self.get_coresight_ap_address(self.mem_ap_ix, 0 * 4),
-                data=0x22000012,
-            )
-            self.coresight_reg_write(
-                access_port=True,
-                addr=self.get_coresight_ap_address(self.mem_ap_ix, 1 * 4),
-                data=addr,
-            )
-            self.coresight_reg_write(
-                access_port=True,
-                addr=self.get_coresight_ap_address(self.mem_ap_ix, 3 * 4),
-                data=data,
-            )
-        except SPSDKError as exc:
-            self.clear_sticky_errors()
-            raise SPSDKDebugProbeTransferError(f"Failed write memory({str(exc)})") from exc
-
-    @classmethod
-    def get_coresight_ap_address(cls, access_port: int, address: int) -> int:
-        """Return computed address of coresight access port register.
-
-        :param access_port: Index of access port 0-255.
-        :param address: Register address.
-        :return: Coresight address.
-        :raises SPSDKError: In case of invalid value.
-        """
-        if access_port > 255:
-            raise SPSDKValueError("Invalid value of access port")
-
-        return access_port << cls.APSEL_SHIFT | address
-
-    def coresight_reg_read(self, access_port: bool = True, addr: int = 0) -> int:
-        """Read coresight register.
-
-        It reads coresight register function for SPSDK library to support various DEBUG PROBES.
-
-        :param access_port: if True, the Access Port (AP) register will be read(default), otherwise the Debug Port
-        :param addr: the register address
-        :return: The read value of addressed register (4 bytes)
-        :raises NotImplementedError: Derived class has to implement this method
-        """
-        raise NotImplementedError("Derived class has to implement this method.")
-
-    def coresight_reg_write(self, access_port: bool = True, addr: int = 0, data: int = 0) -> None:
-        """Write coresight register.
-
-        It writes coresight register function for SPSDK library to support various DEBUG PROBES.
-
-        :param access_port: if True, the Access Port (AP) register will be write(default), otherwise the Debug Port
-        :param addr: the register address
-        :param data: the data to be written into register
-        :raises NotImplementedError: Derived class has to implement this method
-        """
-        raise NotImplementedError("Derived class has to implement this method.")
+        return self._mem_reg_write(mem_ap_ix=self.mem_ap_ix, addr=addr, data=data)
 
     def clear_sticky_errors(self) -> None:
         """Clear sticky errors of Debug port interface."""
+        if self.options.get("use_jtag") is not None:
+            # Currently clear_sticky_errors has been defined only for SWD (uncleared for JTAG-DP)
+            self.coresight_reg_write(access_port=False, addr=4, data=0x50000F20)
         disable_reinit = self.disable_reinit
         try:
             self.disable_reinit = True
@@ -384,24 +495,6 @@ class DebugProbe:
                 f"Bank: {hex((self.last_accessed_ap & self.APBANKSEL) >> self.APBANK_SHIFT)}"
             )
 
-    def assert_reset_line(self, assert_reset: bool = False) -> None:
-        """Control reset line at a target.
-
-        :param assert_reset: If True, the reset line is asserted(pulled down), if False the reset line is not affected.
-        :raises NotImplementedError: Derived class has to implement this method
-        """
-        raise NotImplementedError("Derived class has to implement this method.")
-
-    def reset(self) -> None:
-        """Reset a target.
-
-        It resets a target.
-        """
-        self.assert_reset_line(True)
-        sleep(self.RESET_TIME)
-        self.assert_reset_line(False)
-        sleep(self.AFTER_RESET_TIME)
-
     def __del__(self) -> None:
         """General Debug Probe 'END' event handler."""
         try:
@@ -409,24 +502,62 @@ class DebugProbe:
         except NotImplementedError:
             pass
 
-    def get_ap_list(self, ap_filter: Optional[List[int]] = None) -> Dict[int, int]:
-        """Gets the dictionary of AP IDR's active in target.
 
-        :param ap_filter: List of AP be scanned - otherwise all range will be used[0-255].
-        :return: Dictionary with active AP's. Key is index of AP, value is IDR value.
+class ProbeDescription:
+    """NamedTuple for DAT record of debug probe description."""
+
+    def __init__(
+        self,
+        interface: str,
+        hardware_id: str,
+        description: str,
+        probe: Type[DebugProbe],
+    ) -> None:
+        """Initialization of Debug probe description class.
+
+        param interface: Probe Interface.
+        param hardware_id: Probe Hardware ID(Identification).
+        param description: Probe Text description.
+        param probe: Probe name of the class.
         """
-        ret: Dict[int, int] = {}
-        for i in ap_filter or range(256):
-            try:
-                idr = self.coresight_reg_read(
-                    access_port=True,
-                    addr=self.get_coresight_ap_address(access_port=i, address=self.IDR_REG),
-                )
-            except SPSDKError:
-                pass
-            else:
-                if idr != 0:
-                    logger.debug(f"Find AP{i} with IDR value:{hex(idr)}")
-                    ret[i] = idr
+        self.interface = interface
+        self.hardware_id = hardware_id
+        self.description = description
+        self.probe = probe
 
-        return ret
+    def get_probe(self, options: Optional[Dict] = None) -> DebugProbe:
+        """Get instance of probe.
+
+        :param options: The dictionary with options
+        :return: Instance of described probe.
+        """
+        return self.probe(hardware_id=self.hardware_id, options=options)
+
+    def __str__(self) -> str:
+        """Provide string representation of debug probe."""
+        return f"Debug probe: {self.interface}; {self.description}. S/N:{self.hardware_id}"
+
+
+class DebugProbes(List[ProbeDescription]):
+    """Helper class for debug probe selection. This class accepts only ProbeDescription object."""
+
+    def __str__(self) -> str:
+        """Prints the List of Probes to nice colored table."""
+        table = prettytable.PrettyTable(["#", "Interface", "Id", "Description"])
+        table.align = "l"
+        table.header = True
+        table.border = True
+        table.hrules = prettytable.HEADER
+        table.vrules = prettytable.NONE
+        i = 0
+        for probe in self:
+            table.add_row(
+                [
+                    colorama.Fore.YELLOW + str(i),
+                    colorama.Fore.WHITE + probe.interface,
+                    colorama.Fore.CYAN + probe.hardware_id,
+                    colorama.Fore.GREEN + probe.description,
+                ]
+            )
+            i += 1
+        return table.get_string() + colorama.Style.RESET_ALL
