@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2023 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,7 +11,8 @@
 from struct import pack, unpack, unpack_from
 from typing import Dict, List, Optional, Type
 
-from spsdk.utils.easy_enum import Enum
+from spsdk.utils.interfaces.commands import CmdPacketBase, CmdResponseBase
+from spsdk.utils.spsdk_enum import SpsdkEnum
 
 from .error_codes import StatusCode
 from .exceptions import McuBootError
@@ -21,7 +22,7 @@ from .exceptions import McuBootError
 ########################################################################################################################
 
 # fmt: off
-class CommandTag(Enum):
+class CommandTag(SpsdkEnum):
     """McuBoot Commands."""
 
     NO_COMMAND                  = (0x00, "NoCommand", "No Command")
@@ -58,7 +59,7 @@ class CommandTag(Enum):
 
 
 
-class CommandFlag(Enum):
+class CommandFlag(SpsdkEnum):
     """Flags for McuBoot commands."""
 
     NONE           = (0, "NoFlags", "No flags specified")
@@ -66,7 +67,7 @@ class CommandFlag(Enum):
 
 
 
-class ResponseTag(Enum):
+class ResponseTag(SpsdkEnum):
     """McuBoot Responses to Commands."""
 
     GENERIC                     = (0xA0, "GenericResponse", "Generic Response")
@@ -79,7 +80,7 @@ class ResponseTag(Enum):
     TRUST_PROVISIONING_RESPONSE = (0xB6, "TrustProvisioningResponse", "Trust Provisioning Response")
 
 
-class KeyProvOperation(Enum):
+class KeyProvOperation(SpsdkEnum):
     """Type of key provisioning operation."""
 
     ENROLL              = (0, "Enroll", "Enroll Operation")
@@ -91,21 +92,21 @@ class KeyProvOperation(Enum):
     READ_KEY_STORE      = (6, "ReadKeyStore", "Read Key Store Operation")
 
 
-class KeyProvUserKeyType(Enum):
+class KeyProvUserKeyType(SpsdkEnum):
     """Enumeration of supported user keys in PUF. Keys are SoC specific, not all will be supported for the processor."""
 
-    OTFADKEK        = (2, "OTFADKEK", "Key for OTFAD encryption")  # used on RTxxx
-    SBKEK           = (3, "SBKEK", "Key for SB file encryption")  # Available on LPC55Sxx and RTxxx
-    PRINCE_REGION_0 = (7, "PRINCE0", "Key for Prince region 0")  # LPC55Sxx
-    PRINCE_REGION_1 = (8, "PRINCE1", "Key for Prince region 1")  # LPC55Sxx
-    PRINCE_REGION_2 = (9, "PRINCE2", "Key for Prince region 2")  # LPC55Sxx
-    PRINCE_REGION_3 = (10, "PRINCE3", "Key for Prince region 3")  # NHS52xx
+    OTFADKEK        = (2, "OTFADKEK", "Key for OTFAD encryption")
+    SBKEK           = (3, "SBKEK", "Key for SB file encryption")
+    PRINCE_REGION_0 = (7, "PRINCE0", "Key for Prince region 0")
+    PRINCE_REGION_1 = (8, "PRINCE1", "Key for Prince region 1")
+    PRINCE_REGION_2 = (9, "PRINCE2", "Key for Prince region 2")
+    PRINCE_REGION_3 = (10, "PRINCE3", "Key for Prince region 3")
 
-    USERKEK         = (11, "USERKEK", "Encrypted boot image key")  # LPC55Sxx and RTxxx
-    UDS             = (12, "UDS", "Universal Device Secret for DICE")  # LPC55Sxx and RTxxx
+    USERKEK         = (11, "USERKEK", "Encrypted boot image key")
+    UDS             = (12, "UDS", "Universal Device Secret for DICE")
 
 
-class GenerateKeyBlobSelect(Enum):
+class GenerateKeyBlobSelect(SpsdkEnum):
     """Key selector for the generate-key-blob function.
 
     For devices with SNVS, valid options of [key_sel] are
@@ -120,7 +121,7 @@ class GenerateKeyBlobSelect(Enum):
     CMK     = (3, "CMK", "CMK from SNVS")
 
 
-class TrustProvOperation(Enum):
+class TrustProvOperation(SpsdkEnum):
     """Operations supported by Trust Provisioning flow."""
 
     PROVE_GENUINITY = (0xF4, "ProveGenuinity", "Start the proving genuinity process")
@@ -136,7 +137,7 @@ class TrustProvOperation(Enum):
     HSM_ENC_SIGN                = (6, "HsnEncSign", "HSM enc sign")
 
 
-class TrustProvOemKeyType(Enum):
+class TrustProvOemKeyType(SpsdkEnum):
     """Type of oem key type definition."""
 
     MFWISK      = (0xC3A5, "MFWISK", "ECDSA Manufacturing Firmware Signing Key")
@@ -145,7 +146,7 @@ class TrustProvOemKeyType(Enum):
     GETCUSTMKSK = (0x3C5A, "GETCUSTMKSK", "CKDF Master Key for Production Firmware Encryption Key")
 
 
-class TrustProvKeyType(Enum):
+class TrustProvKeyType(SpsdkEnum):
     """Type of key type definition."""
 
     CKDFK = (1, "CKDFK", "CKDF Master Key")
@@ -156,12 +157,27 @@ class TrustProvKeyType(Enum):
     KUOK  = (6, "KUOK", "Key Unwrap Only Key")
 
 
-class TrustProvWrappingKeyType(Enum):
+class TrustProvWrappingKeyType(SpsdkEnum):
     """Type of wrapping key type definition."""
 
     INT_SK = (0x10, "INT_SK", "The wrapping key for wrapping of MFG_CUST_MK_SK0_BLOB")
     EXT_SK = (0x11, "EXT_SK", "The wrapping key for wrapping of MFG_CUST_MK_SK0_BLOB")
 
+class TrustProvWpc(SpsdkEnum):
+    """Type of WPC trusted facility commands for DSC."""
+
+    WPC_GET_ID              = (0x5000000, "wpc_get_id", "WPC get ID")
+    NXP_GET_ID              = (0x5000001, "nxp_get_id", "NXP get ID")
+    WPC_INSERT_CERT         = (0x5000002, "wpc_insert_cert", "WPC insert certificate")
+    WPC_SIGN_CSR            = (0x5000003, "wpc_sign_csr", "WPC sign CSR")
+
+class TrustProvDevHsmDsc(SpsdkEnum):
+    """Type of DSC Device HSM."""
+
+    DSC_HSM_CREATE_SESSION  = (0x6000000, "dsc_hsm_create_session", "DSC HSM create session")
+    DSC_HSM_ENC_BLK         = (0x6000001, "dsc_hsm_enc_blk", "DSC HSM encrypt bulk")
+    DSC_HSM_ENC_SIGN        = (0x6000002, "dsc_hsm_enc_sign", "DSC HSM sign")
+
 # fmt: on
 
 ########################################################################################################################
@@ -193,10 +209,10 @@ class CmdHeader:
     def __ne__(self, obj: object) -> bool:
         return not self.__eq__(obj)
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"<Tag=0x{self.tag:02X}, Flags=0x{self.flags:02X}, ParamsCount={self.params_count}>"
 
-    def __repr__(self) -> str:
+    def __str__(self) -> str:
         return (
             f"CmdHeader(tag=0x{self.tag:02X}, flags=0x{self.flags:02X}, "
             f"reserved={self.reserved}, params_count={self.params_count})"
@@ -220,13 +236,15 @@ class CmdHeader:
         return cls(*unpack_from("4B", data, offset))
 
 
-class CmdPacket:
+class CmdPacket(CmdPacketBase):
     """McuBoot command packet format class."""
 
     SIZE = 32
     EMPTY_VALUE = 0x00
 
-    def __init__(self, tag: int, flags: int, *args: int, data: Optional[bytes] = None) -> None:
+    def __init__(
+        self, tag: CommandTag, flags: int, *args: int, data: Optional[bytes] = None
+    ) -> None:
         """Initialize the Command Packet object.
 
         :param tag: Tag identifying the command
@@ -234,7 +252,7 @@ class CmdPacket:
         :param args: Arguments used by the command
         :param data: Additional data, defaults to None
         """
-        self.header = CmdHeader(tag, flags, 0, len(args))
+        self.header = CmdHeader(tag.tag, flags, 0, len(args))
         self.params = list(args)
         if data is not None:
             if len(data) % 4:
@@ -249,11 +267,12 @@ class CmdPacket:
         return not self.__eq__(obj)
 
     def __str__(self) -> str:
-        return "<" + self.info() + ">"
-
-    def info(self) -> str:
         """Get object info."""
-        tag = CommandTag.get(self.header.tag, f"0x{self.header.tag:02X}")
+        tag = (
+            CommandTag.get_label(self.header.tag)
+            if self.header.tag in CommandTag.tags()
+            else f"0x{self.header.tag:02X}"
+        )
         return f"Tag={tag}, Flags=0x{self.header.flags:02X}" + "".join(
             f", P[{n}]=0x{param:08X}" for n, param in enumerate(self.params)
         )
@@ -272,7 +291,7 @@ class CmdPacket:
         return data
 
 
-class CmdResponse:
+class CmdResponse(CmdResponseBase):
     """McuBoot response base format class."""
 
     def __init__(self, header: CmdHeader, raw_data: bytes) -> None:
@@ -288,6 +307,18 @@ class CmdResponse:
         (status,) = unpack_from("<L", raw_data)
         self.status: int = status
 
+    @property
+    def value(self) -> int:
+        """Return a integer representation of the response."""
+        return unpack_from(">I", self.raw_data)[0]
+
+    def _get_status_label(self) -> str:
+        return (
+            StatusCode.get_label(self.status)
+            if self.status in StatusCode.tags()
+            else f"Unknown[0x{self.status:08X}]"
+        )
+
     def __eq__(self, obj: object) -> bool:
         return isinstance(obj, CmdResponse) and vars(obj) == vars(self)
 
@@ -295,9 +326,6 @@ class CmdResponse:
         return not self.__eq__(obj)
 
     def __str__(self) -> str:
-        return "<" + self.info() + ">"
-
-    def info(self) -> str:
         """Get object info."""
         return (
             f"Tag=0x{self.header.tag:02X}, Flags=0x{self.header.flags:02X}"
@@ -320,11 +348,15 @@ class GenericResponse(CmdResponse):
         _, tag = unpack_from("<2I", raw_data)
         self.cmd_tag: int = tag
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
-        cmd = CommandTag.get(self.cmd_tag, f"Unknown[0x{self.cmd_tag:02X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
+        cmd = (
+            CommandTag.get_label(self.cmd_tag)
+            if self.cmd_tag in CommandTag.tags()
+            else f"Unknown[0x{self.cmd_tag:02X}]"
+        )
         return f"Tag={tag}, Status={status}, Cmd={cmd}"
 
 
@@ -341,10 +373,10 @@ class GetPropertyResponse(CmdResponse):
         _, *values = unpack_from(f"<{self.header.params_count}I", raw_data)
         self.values: List[int] = list(values)
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}" + "".join(
             f", v{n}=0x{value:08X}" for n, value in enumerate(self.values)
         )
@@ -363,10 +395,10 @@ class ReadMemoryResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -385,10 +417,10 @@ class FlashReadOnceResponse(CmdResponse):
         self.values: List[int] = list(values)
         self.data = raw_data[8 : 8 + self.length] if self.length > 0 else b""
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -405,10 +437,10 @@ class FlashReadResourceResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -425,10 +457,10 @@ class KeyProvisioningResponse(CmdResponse):
         _, length = unpack_from("<2I", raw_data)
         self.length: int = length
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}, Length={self.length}"
 
 
@@ -445,10 +477,10 @@ class TrustProvisioningResponse(CmdResponse):
         _, *values = unpack(f"<{self.header.params_count}I", raw_data)
         self.values: List[int] = list(values)
 
-    def info(self) -> str:
+    def __str__(self) -> str:
         """Get object info."""
-        tag = ResponseTag.name(self.header.tag)
-        status = StatusCode.get(self.status, f"Unknown[0x{self.status:08X}]")
+        tag = ResponseTag.get_label(self.header.tag)
+        status = self._get_status_label()
         return f"Tag={tag}, Status={status}"
 
 
@@ -461,7 +493,7 @@ class NoResponse(CmdResponse):
         :param cmd_tag: Tag of the command that preceded the no-response from target
         """
         header = CmdHeader(tag=cmd_tag, flags=0, reserved=0, params_count=0)
-        raw_data = pack("<L", StatusCode.NO_RESPONSE)
+        raw_data = pack("<L", StatusCode.NO_RESPONSE.tag)
         super().__init__(header, raw_data)
 
 
@@ -473,14 +505,14 @@ def parse_cmd_response(data: bytes, offset: int = 0) -> CmdResponse:
     :return: De-serialized object from data
     """
     known_response: Dict[int, Type[CmdResponse]] = {
-        ResponseTag.GENERIC: GenericResponse,
-        ResponseTag.GET_PROPERTY: GetPropertyResponse,
-        ResponseTag.READ_MEMORY: ReadMemoryResponse,
-        ResponseTag.FLASH_READ_RESOURCE: FlashReadResourceResponse,
-        ResponseTag.FLASH_READ_ONCE: FlashReadOnceResponse,
-        ResponseTag.KEY_BLOB_RESPONSE: ReadMemoryResponse,  # not sure what format is returned, this work on RT1050
-        ResponseTag.KEY_PROVISIONING_RESPONSE: KeyProvisioningResponse,
-        ResponseTag.TRUST_PROVISIONING_RESPONSE: TrustProvisioningResponse,
+        ResponseTag.GENERIC.tag: GenericResponse,
+        ResponseTag.GET_PROPERTY.tag: GetPropertyResponse,
+        ResponseTag.READ_MEMORY.tag: ReadMemoryResponse,
+        ResponseTag.FLASH_READ_RESOURCE.tag: FlashReadResourceResponse,
+        ResponseTag.FLASH_READ_ONCE.tag: FlashReadOnceResponse,
+        ResponseTag.KEY_BLOB_RESPONSE.tag: ReadMemoryResponse,
+        ResponseTag.KEY_PROVISIONING_RESPONSE.tag: KeyProvisioningResponse,
+        ResponseTag.TRUST_PROVISIONING_RESPONSE.tag: TrustProvisioningResponse,
     }
     header = CmdHeader.from_bytes(data, offset)
     if header.tag in known_response:
