@@ -1,32 +1,24 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """The shadow registers control DAT support file."""
 
 import logging
-from typing import Any, List, Optional
+from typing import Any, Dict, List, Optional
 
-from ruamel.yaml import YAML
-from ruamel.yaml.comments import CommentedMap as CM
-
-from spsdk import SPSDK_YML_INDENT, SPSDKError, __author__, __release__, __version__
+from spsdk import __author__, __release__, __version__
 from spsdk.dat.debug_mailbox import DebugMailbox
 from spsdk.dat.dm_commands import StartDebugSession
 from spsdk.debuggers.debug_probe import DebugProbe, SPSDKDebugProbeError
 from spsdk.debuggers.utils import test_ahb_access
-from spsdk.utils.misc import (
-    change_endianness,
-    load_configuration,
-    value_to_bytes,
-    value_to_int,
-    write_file,
-)
+from spsdk.exceptions import SPSDKError
+from spsdk.utils.database import DatabaseManager, get_db, get_families, get_schema_file
+from spsdk.utils.misc import Endianness
 from spsdk.utils.reg_config import RegConfig
 from spsdk.utils.registers import Registers, RegsRegister, SPSDKRegsErrorRegisterNotFound
-from spsdk.utils.schema_validator import ConfigTemplate
 
 logger = logging.getLogger(__name__)
 
@@ -42,35 +34,28 @@ class ShadowRegisters:
         self,
         debug_probe: Optional[DebugProbe],
         config: RegConfig,
-        device: str,
-        revision: str = "latest",
     ) -> None:
         """Initialization of Shadow register class."""
         self.probe = debug_probe
         self.config = config
-        self.device = device
-        self.offset = value_to_int(self.config.get_address(self.device))
+        self.device = config.family
+        self.offset = self.config.get_address()
+        self.offset_for_read = self.config.get_address(alt_read_address=True)
         self.fuse_mode = False
-
         self.regs = Registers(self.device)
-        rev = revision or "latest"
-        self.revision = (
-            rev
-            if rev != "latest"
-            else config.devices.get_by_name(self.device).revisions.get_latest().name
-        )
+
         self.regs.load_registers_from_xml(
-            config.get_data_file(self.device, self.revision),
-            grouped_regs=config.get_grouped_registers(self.device),
+            config.get_data_file(),
+            grouped_regs=config.get_grouped_registers(),
         )
 
         # Set the computed field handler
-        for reg, fields in self.config.get_computed_fields(self.device).items():
+        for reg, fields in self.config.get_computed_fields().items():
             reg_obj = self.regs.find_reg(reg)
             reg_obj.add_setvalue_hook(self.reg_computed_fields_handler, fields)
 
         # Set the antipolize handler
-        for reg, antipole_reg in self.config.get_antipole_regs(self.device).items():
+        for reg, antipole_reg in self.config.get_antipole_regs().items():
             src = self.regs.find_reg(reg)
             dst = self.regs.find_reg(antipole_reg)
             src.add_setvalue_hook(self.reg_antipolize_src_handler, dst)
@@ -91,7 +76,7 @@ class ShadowRegisters:
                 "Shadow registers: Cannot use the communication function without defined debug probe."
             )
 
-        logger.debug(f"Writing shadow register address: {hex(addr)}, data: {hex(data)}")
+        logger.info(f"Writing shadow register address: {hex(addr)}, data: {hex(data)}")
         self.probe.mem_reg_write(addr, data)
 
         if verify_mask:
@@ -105,7 +90,7 @@ class ShadowRegisters:
     def reload_registers(self) -> None:
         """Reload all the values in managed registers."""
         for reg in self.regs.get_registers():
-            self.reload_register(reg)
+            self.get_register(reg.name)
 
     def sets_all_registers(self, verify: bool = True) -> None:
         """Update all shadow registers in target by local values.
@@ -113,33 +98,20 @@ class ShadowRegisters:
         :param verify: Verity write operation.
         """
         for reg in self.regs.get_registers():
-            self.set_register(reg.name, reg.get_value(), verify)
+            self.set_register(reg.name, reg.get_value(raw=True), verify)
 
-    def reload_register(self, reg: RegsRegister) -> None:
-        """Reload the value in requested register.
-
-        :param reg: The register to reload from the HW.
-        """
-        reg.set_value(self.get_register(reg.name), raw=True)
-
-    def read_register(self, reg: RegsRegister) -> bytes:
-        """Read the value in requested register.
-
-        :param reg: The register to read from the HW.
-        """
-        return bytes(self.get_register(reg.name))
-
-    def set_register(self, reg_name: str, data: Any, verify: bool = True) -> None:
+    def set_register(self, reg_name: str, data: Any, verify: bool = True, raw: bool = True) -> None:
         """The function sets the value of the specified register.
 
         :param reg_name: The register name.
         :param data: The new data to be stored to shadow register.
         :param verify: Verity write operation.
+        :param raw: Do not use any modification hooks.
         :raises SPSDKDebugProbeError: The debug probe is not specified.
         :raises SPSDKError: General error with write of Shadow register.
         """
 
-        def write_reg(base_address: int, reg: RegsRegister) -> None:
+        def write_reg(reg: RegsRegister) -> None:
             if not self.probe:
                 raise SPSDKDebugProbeError(
                     "Shadow registers: Cannot use the communication function without defined debug probe."
@@ -149,10 +121,6 @@ class ShadowRegisters:
                 raise SPSDKError(
                     f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to write to device."
                 )
-            # Create value
-            value = reg.get_value()
-            if reg.reverse:
-                value = int.from_bytes(change_endianness(value_to_bytes(value)), "big")
             # Create verify mask
             verify_mask = 0
             if verify:
@@ -164,18 +132,20 @@ class ShadowRegisters:
                     verify_mask = (1 << reg.width) - 1
 
             self._write_shadow_reg(
-                addr=base_address + reg.offset, data=value, verify_mask=verify_mask
+                addr=self.offset + reg.offset,
+                data=reg.get_value(raw=True),
+                verify_mask=verify_mask,
             )
 
         try:
             reg = self.regs.find_reg(reg_name, include_group_regs=True)
-            reg.set_value(data, raw=True)
+            reg.set_value(data, raw)
             if reg.has_group_registers():
-                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
-                    write_reg(self.offset, sub_reg)
+                for sub_reg in reg.sub_regs:
+                    write_reg(sub_reg)
             else:
-                write_reg(self.offset, reg=reg)
-            # execute flash function handler if defined for a platfrom
+                write_reg(reg)
+            # execute flash function handler if defined for a platform
             self.flush_func_handler()
         except SPSDKError as exc:
             raise SPSDKError(f"The set shadow register failed({str(exc)}).") from exc
@@ -188,7 +158,7 @@ class ShadowRegisters:
         raises SPSDKDebugProbeError: The debug probe is not specified.
         """
 
-        def read_reg(base_address: int, reg: RegsRegister) -> bytes:
+        def read_reg(reg: RegsRegister) -> None:
             if not self.probe:
                 raise SPSDKDebugProbeError(
                     "Shadow registers: Cannot use the communication function without defined debug probe."
@@ -197,21 +167,17 @@ class ShadowRegisters:
                 raise SPSDKError(
                     f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to read from device."
                 )
-            read_value = self.probe.mem_reg_read(base_address + reg.offset).to_bytes(4, "big")
-            if reg.reverse:
-                read_value = change_endianness(read_value)
-            return read_value
+            reg.set_value(self.probe.mem_reg_read(self.offset_for_read + reg.offset), raw=True)
 
         try:
             reg = self.regs.find_reg(reg_name, include_group_regs=True)
 
-            ret = bytearray()
             if reg.has_group_registers():
-                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
-                    ret.extend(read_reg(self.offset, sub_reg))
+                for sub_reg in reg.sub_regs:
+                    read_reg(sub_reg)
             else:
-                ret.extend(read_reg(self.offset, reg=reg))
-            return ret
+                read_reg(reg)
+            return reg.get_bytes_value(raw=True)
 
         except SPSDKError as exc:
             raise SPSDKError(f"The get shadow register failed({str(exc)}).") from exc
@@ -227,7 +193,7 @@ class ShadowRegisters:
         def add_reg(reg: RegsRegister) -> str:
             otp_index = reg.otp_index
             assert otp_index
-            otp_value = "0x" + reg.get_bytes_value().hex()
+            otp_value = "0x" + reg.get_bytes_value(raw=True).hex()
             burn_fuse = f"# Fuse {reg.name}, index {otp_index} and value: {otp_value}.\n"
             burn_fuse += f"efuse-program-once {hex(otp_index)} {otp_value}\n"
             return burn_fuse
@@ -235,10 +201,10 @@ class ShadowRegisters:
         ret = (
             "# BLHOST fuses programming script\n"
             f"# Generated by SPSDK {__version__}\n"
-            f"# Chip: {self.device} rev:{self.revision}\n\n\n"
+            f"# Chip: {self.device} rev:{self.config.revision}\n\n\n"
         )
         # Update list by antipole opposites registers
-        for ap_reg_src, ap_reg_dst in self.config.get_antipole_regs(self.device).items():
+        for ap_reg_src, ap_reg_dst in self.config.get_antipole_regs().items():
             if ap_reg_src in reg_list:
                 reg_list.insert(reg_list.index(ap_reg_src) + 1, ap_reg_dst)
         self.fuse_mode = True
@@ -262,58 +228,68 @@ class ShadowRegisters:
         self.fuse_mode = False
         return ret
 
-    def create_yaml_config(self, file_name: str, raw: bool = False, diff: bool = False) -> None:
-        """The function creates the configuration YML file.
+    @staticmethod
+    def get_supported_families() -> List[str]:
+        """Return list of supported families."""
+        return get_families(DatabaseManager.SHADOW_REGS)
 
-        :param file_name: The file_name (without extension) of stored configuration.
-        :param raw: Raw output of configuration (including computed fields and anti-pole registers)
-        :param diff: Get only configuration with difference value to reset state.
+    @classmethod
+    def get_validation_schemas_family(cls) -> List[Dict[str, Any]]:
+        """Create the validation schema just for supported families.
+
+        :return: List of validation schemas for Shadow registers supported families.
         """
-        antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
-        computed_fields = None if raw else self.config.get_computed_fields(self.device)
+        sch_cfg = get_schema_file(DatabaseManager.SHADOW_REGS)
+        sch_cfg["sr_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
+        return [sch_cfg["sr_family_rev"]]
 
-        yaml = YAML()
-        yaml.indent(sequence=SPSDK_YML_INDENT * 2, offset=SPSDK_YML_INDENT)
-        data = CM()
+    @classmethod
+    def get_validation_schemas(cls, family: str, revision: str = "latest") -> List[Dict[str, Any]]:
+        """Create the validation schema.
 
-        description = CM()
-        description.yaml_set_start_comment(
-            f"NXP {self.device.upper()} Shadow registers configuration", indent=2
-        )
-        description.insert(1, "device", self.device, comment="The NXP device name.")
-        description.insert(
-            2, "version", __version__, comment="The SPSDK Shadow register tool version."
-        )
-        description.insert(3, "author", __author__, comment="The author of the configuration.")
-        description.insert(4, "release", __release__, comment="The SPSDK release.")
-
-        data["description"] = description
-        data["registers"] = self.regs.create_yml_config(
-            exclude_regs=antipole_regs,
-            exclude_fields=computed_fields,
-            indent=2,
-            diff=diff,
-        )
-        write_file(ConfigTemplate.convert_cm_to_yaml(data), file_name, encoding="utf8")
-
-    def load_yaml_config(self, file_name: str, raw: bool = False) -> None:
-        """The function loads the configuration from YML file.
-
-        :param file_name: The file_name (without extension) of stored configuration.
-        :param raw: Raw input of configuration (including computed fields and anti-pole registers)
-        :raises SPSDKError: When the configuration file not found.
+        :param family: Family description.
+        :param revision: Chip revision specification, as default, latest is used.
+        :raises SPSDKError: Family or revision is not supported.
+        :return: List of validation schemas.
         """
-        antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
-        computed_fields = None if raw else self.config.get_computed_fields(self.device)
-        data = load_configuration(file_name)
-        self.regs.load_yml_config(data["registers"], antipole_regs, computed_fields)
-        if not raw:
-            # Just update only configured registers
-            exclude_hooks = list(set(self.regs.get_reg_names()) - set(data["registers"].keys()))
-            self.regs.run_hooks(exclude_hooks)
+        sch_cfg = get_schema_file(DatabaseManager.SHADOW_REGS)
+        try:
+            db = get_db(family, revision)
+            regs = Registers(family, Endianness.LITTLE)
+            regs.load_registers_from_xml(
+                db.get_file_path(DatabaseManager.SHADOW_REGS, "data_file"),
+                grouped_regs=db.get_list(DatabaseManager.SHADOW_REGS, "grouped_registers"),
+            )
+            sch_cfg["sr_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
+            sch_cfg["sr_family_rev"]["properties"]["family"]["template_value"] = family
+            sch_cfg["sr_family_rev"]["properties"]["revision"]["template_value"] = revision
+            sch_cfg["sr_registers"]["properties"]["registers"][
+                "properties"
+            ] = regs.get_validation_schema()["properties"]
+            return [sch_cfg["sr_family_rev"], sch_cfg["sr_registers"]]
+        except (KeyError, SPSDKError) as exc:
+            raise SPSDKError(f"Family {family} or revision {revision} is not supported") from exc
+
+    def load_config(self, config: Dict[str, Any]) -> None:
+        """The function loads the configuration.
+
+        :param config: The configuration of shadow registers.
+        """
+        self.regs.load_yml_config(config["registers"])
 
         logger.debug("The shadow registers has been loaded from configuration.")
 
+    def get_config(self, diff: bool = False) -> Dict[str, Any]:
+        """The function creates the configuration.
+
+        :param diff: If sett only changed registers will be placed in configuration.
+        """
+        ret: Dict[str, Any] = {"family": self.config.family, "revision": self.config.revision}
+        ret["registers"] = self.regs.get_config(diff)
+
+        logger.debug("The shadow registers creates configuration.")
+        return ret
+
     @staticmethod
     def reg_antipolize_src_handler(val: int, context: Any) -> int:
         """Antipolize given register value.
@@ -345,7 +321,7 @@ class ShadowRegisters:
         :param self: Input Value.
         :raises SPSDKError: Raises when the computing routine is not found.
         """
-        flush_func = self.config.get_device_value("flush_func", device=self.device)
+        flush_func = self.config.get_value("flush_func", "")
         if flush_func:
             if hasattr(self, flush_func):
                 method_ref = getattr(self, flush_func)
@@ -425,7 +401,7 @@ class ShadowRegisters:
         return val | 0x80000000
 
     def rw61x_update_scratch_reg(self) -> None:
-        """Function updates scracth register for RW61x, This enables the shadow register functionality.
+        """Function updates scratch register for RW61x, This enables the shadow register functionality.
 
         :param self: Input Value.
         """
