@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2021-2022 NXP
+# Copyright 2021-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Trust provisioning adapters."""
 
 from typing import Dict, Type
 
-from .. import TpDevInterface, TpTargetInterface
+from ..tp_intf import TpDevInterface, TpTargetInterface
 from .tpdev_scard import TpDevSmartCard
 from .tptarget_blhost import TpTargetBlHost
 
