@@ -1,16 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2021-2023 NXP
+# Copyright 2021-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for generating, processing and verifying TP Audit Log."""
 import contextlib
 import os
 import sqlite3
-from typing import Iterator, List, NamedTuple, Optional, Tuple, Union
+from typing import Iterator, List, NamedTuple, Optional, Tuple
 
-from spsdk.utils.crypto.backend_openssl import ec, openssl_backend
+from spsdk.crypto.hash import get_hash
+from spsdk.crypto.keys import PublicKeyEcc
+from spsdk.utils.misc import Endianness
 
 from ..exceptions import SPSDKTpError
 from .data_container import Container
@@ -151,7 +153,7 @@ class AuditLogRecord(NamedTuple):
     @property
     def prod_counter_int(self) -> int:
         """Return production counter as an integer."""
-        return int.from_bytes(self.prod_counter, byteorder="big")
+        return int.from_bytes(self.prod_counter, byteorder=Endianness.BIG.value)
 
     def as_dict(self) -> dict:
         """Return dictionary suitable for writing into log file."""
@@ -196,17 +198,16 @@ class AuditLogRecord(NamedTuple):
             data_to_hash += oem_cert or bytes()
         data_to_hash += self.prod_counter
         data_to_hash += self.start_hash
-        return openssl_backend.hash(data_to_hash)
+        return get_hash(data_to_hash)
 
-    def is_valid(self, key: Union[ec.EllipticCurvePublicKey, bytes]) -> bool:
+    def is_valid(self, key: PublicKeyEcc) -> bool:
         """Check if record is valid.
 
         :param key: PEM-encoded public key or public key object
         :return: True if signature checks out
         """
         new_hash = self.new_hash()
-        return openssl_backend.ecc_verify(
-            public_key=key,
+        return key.verify_signature(
             signature=self.signature,
             data=new_hash,
         )
